@@ -1,6 +1,6 @@
 /*
  * Copyright 1993-1995 Bill Triggs <Bill.Triggs@inrialpes.fr>
- * Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+ * Copyright 1995-2021 Bruno Haible <bruno@clisp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _AVCALL_INTERNAL_H
@@ -85,6 +85,8 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #endif
 #if defined(__hppa__) && !defined(__hppa64__)
 #define __av_start1(LIST,LIST_ARGS_END)					\
+   (LIST).farg_mask = 0,						\
+   (LIST).darg_mask = 0,						\
    (LIST).aptr = (LIST).args_end = (LIST_ARGS_END),
 #endif
 #if defined(__hppa64__)
@@ -109,6 +111,13 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
    (LIST).farg_mask = 0,						\
    (LIST).darg_mask = 0,
 #endif
+#if defined(__riscv32__) || defined(__riscv64__)
+#define __av_start1(LIST,LIST_ARGS_END)					\
+   (LIST).aptr = &(LIST).args[0],					\
+   (LIST).fanum = 0,							\
+   (LIST).farg_mask = 0,						\
+   (LIST).darg_mask = 0,
+#endif
 #if defined(__powerpc_aix__) || defined(__powerpc64__)
 #define __av_start1(LIST,LIST_ARGS_END)					\
    (LIST).aptr = &(LIST).args[0],					\
@@ -212,7 +221,7 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #define __av_start_struct3(LIST)  \
   ((LIST).flags |= __AV_REGISTER_STRUCT_RETURN, 0)
 #endif
-#if defined(__hppa__) && !defined(__hppa64__)
+#if (defined(__hppa__) && !defined(__hppa64__)) || defined(__riscv32__)
 #define __av_reg_struct_return(LIST,TYPE_SIZE,TYPE_SPLITTABLE)  \
   ((TYPE_SIZE) <= 8)
 /* Test __AV_SMALL_STRUCT_RETURN at run time. */
@@ -253,7 +262,7 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #define __av_start_struct3(LIST)  \
   ((LIST).flags |= __AV_REGISTER_STRUCT_RETURN, 0)
 #endif
-#if defined(__hppa64__) || defined(__arm64__) || (defined(__powerpc64__) && defined(__powerpc64_elfv2__)) || defined(__x86_64_sysv__)
+#if defined(__hppa64__) || defined(__arm64__) || (defined(__powerpc64__) && defined(__powerpc64_elfv2__)) || defined(__x86_64_sysv__) || defined(__riscv64__)
 #define __av_reg_struct_return(LIST,TYPE_SIZE,TYPE_SPLITTABLE)  \
   ((TYPE_SIZE) <= 16)
 /* Turn on __AV_REGISTER_STRUCT_RETURN if __AV_SMALL_STRUCT_RETURN was set
@@ -278,24 +287,31 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #endif
 /* Return structure pointer is passed as first arg.
  */
-#if defined(__i386__) || defined(__alpha__) || (defined(__arm__) && !defined(__armhf__)) || defined(__powerpc_aix__) || defined(__powerpc64__)
+#if defined(__i386__) || defined(__alpha__) || (defined(__arm__) && !defined(__armhf__)) || defined(__powerpc_aix__) || defined(__powerpc64__) || defined(__riscv32__) || defined(__riscv64__)
 #define __av_start_struct4(LIST,TYPE_SIZE)				\
-   (*(LIST).aptr++ = (__avword)((LIST).raddr), 0)
+  (*(LIST).aptr++ = (__avword)((LIST).raddr), 0)
 #endif
 #if defined(__armhf__)
 #define __av_start_struct4(LIST,TYPE_SIZE)				\
-   ((LIST).args[(LIST).ianum++] = (__avword)((LIST).raddr), 0)
+  ((LIST).args[(LIST).ianum++] = (__avword)((LIST).raddr), 0)
 #endif
 #if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__) || defined(__sparc64__) || defined(__x86_64_ms__)
 #define __av_start_struct4(LIST,TYPE_SIZE)				\
-   (*(LIST).aptr++ = (__avword)((LIST).raddr),				\
-    (LIST).anum++,							\
-    0									\
-   )
+  (*(LIST).aptr++ = (__avword)((LIST).raddr),				\
+   (LIST).anum++,							\
+   0									\
+  )
 #endif
 #if defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+#if defined(__x86_64_x32__)
+/* The x86_64 ABI, section 10.1, specifies that pointers are zero-extended
+   from 32 bits to 64 bits. */
 #define __av_start_struct4(LIST,TYPE_SIZE)				\
-   ((LIST).iargs[(LIST).ianum++] = (__avword)((LIST).raddr), 0)
+  ((LIST).iargs[(LIST).ianum++] = (unsigned long long)(unsigned long)((LIST).raddr), 0)
+#else
+#define __av_start_struct4(LIST,TYPE_SIZE)				\
+  ((LIST).iargs[(LIST).ianum++] = (__avrword)((LIST).raddr), 0)
+#endif
 #endif
 #endif
 
@@ -314,7 +330,7 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
  * scalar argument types
  */
 
-#if defined(__i386__) || defined(__m68k__) || (defined(__sparc__) && !defined(__sparc64__)) || defined(__alpha__) || defined(__hppa64__) || (defined(__arm__) && !defined(__armhf__)) || defined(__arm64__) || defined(__powerpc__) || defined(__powerpc64__) || defined(__ia64__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+#if defined(__i386__) || defined(__m68k__) || (defined(__sparc__) && !defined(__sparc64__)) || defined(__alpha__) || defined(__hppa64__) || (defined(__arm__) && !defined(__armhf__)) || defined(__arm64__) || defined(__powerpc__) || defined(__powerpc64__) || defined(__ia64__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__) || defined(__riscv32__) || defined(__riscv64__)
 /* Floats and all integer types are passed as words,
  * doubles as two words (on 32-bit platforms) or one word (on 64-bit platforms).
  */
@@ -362,35 +378,94 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 
 #if defined(__arm64__) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
 /* The first __AV_IARG_NUM integer arguments are passed in registers. */
+#define __av_int(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (int)(VAL), 0)			\
+   : __av_word(LIST,(int)(VAL)))
+#else
+#define __av_int(LIST,VAL)	__av_word(LIST,(int)(VAL))
+#endif
+
+#if defined(__arm64__) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+/* The first __AV_IARG_NUM integer arguments are passed in registers. */
+#define __av_uint(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (unsigned int)(VAL), 0)		\
+   : __av_word(LIST,(unsigned int)(VAL)))
+#else
+#define __av_uint(LIST,VAL)	__av_word(LIST,(unsigned int)(VAL))
+#endif
+
+#if (defined(__arm64__) && !(defined(__APPLE__) && defined(__MACH__))) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+/* The first __AV_IARG_NUM integer arguments are passed in registers. */
 #define __av_long(LIST,VAL)						\
   ((LIST).ianum < __AV_IARG_NUM						\
    ? ((LIST).iargs[(LIST).ianum++] = (long)(VAL), 0)			\
    : __av_word(LIST,(long)(VAL)))
+#elif defined(__arm64__) && defined(__APPLE__) && defined(__MACH__)
+#define __av_long(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (long)(VAL), 0)			\
+   : ((LIST).aptr + 2 > (LIST).eptr					\
+      ? -1 :								\
+      ((LIST).aptr += 2,						\
+       ((LIST).aptr)[-2] = (unsigned int)(unsigned long)(long)(VAL),	\
+       ((LIST).aptr)[-1] = (unsigned int)((unsigned long)(long)(VAL) >> 32), \
+       0)))
 #else
 #define __av_long(LIST,VAL)	__av_word(LIST,(long)(VAL))
 #endif
 
-#if defined(__arm64__) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+#if (defined(__arm64__) && !(defined(__APPLE__) && defined(__MACH__))) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
 /* The first __AV_IARG_NUM integer arguments are passed in registers. */
 #define __av_ulong(LIST,VAL)						\
   ((LIST).ianum < __AV_IARG_NUM						\
    ? ((LIST).iargs[(LIST).ianum++] = (unsigned long)(VAL), 0)		\
    : __av_word(LIST,(unsigned long)(VAL)))
+#elif defined(__arm64__) && defined(__APPLE__) && defined(__MACH__)
+#define __av_ulong(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (unsigned long)(VAL), 0)		\
+   : ((LIST).aptr + 2 > (LIST).eptr					\
+      ? -1 :								\
+      ((LIST).aptr += 2,						\
+       ((LIST).aptr)[-2] = (unsigned int)(unsigned long)(VAL),		\
+       ((LIST).aptr)[-1] = (unsigned int)((unsigned long)(VAL) >> 32),	\
+       0)))
 #else
 #define __av_ulong(LIST,VAL)	__av_word(LIST,(unsigned long)(VAL))
 #endif
 
-#if defined(__arm64__) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
+#if (defined(__arm64__) && !(defined(__APPLE__) && defined(__MACH__))) || defined(__powerpc_sysv4__) || defined(__x86_64_sysv__) || defined(__s390__) || defined(__s390x__)
 /* The first __AV_IARG_NUM integer arguments are passed in registers. */
+#if defined(__x86_64_x32__)
+/* The x86_64 ABI, section 10.1, specifies that pointers are zero-extended
+   from 32 bits to 64 bits. */
 #define __av_ptr(LIST,VAL)						\
   ((LIST).ianum < __AV_IARG_NUM						\
-   ? ((LIST).iargs[(LIST).ianum++] = (__avword)(VAL), 0)		\
+   ? ((LIST).iargs[(LIST).ianum++] = (unsigned long long)(unsigned long)(VAL), 0) \
+   : __av_word(LIST,(unsigned long long)(unsigned long)(VAL)))
+#else
+#define __av_ptr(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (__avrword)(VAL), 0)		\
    : __av_word(LIST,VAL))
+#endif
+#elif defined(__arm64__) && defined(__APPLE__) && defined(__MACH__)
+#define __av_ptr(LIST,VAL)						\
+  ((LIST).ianum < __AV_IARG_NUM						\
+   ? ((LIST).iargs[(LIST).ianum++] = (unsigned long)(VAL), 0)		\
+   : ((LIST).aptr + 2 > (LIST).eptr					\
+      ? -1 :								\
+      ((LIST).aptr += 2,						\
+       ((LIST).aptr)[-2] = (unsigned int)(unsigned long)(VAL),		\
+       ((LIST).aptr)[-1] = (unsigned int)((unsigned long)(VAL) >> 32),	\
+       0)))
 #else
 #define __av_ptr(LIST,VAL)	__av_word(LIST,VAL)
 #endif
 
-#if defined(__mips64__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppa64__) || defined(__arm64__) || defined(__powerpc64__) || defined(__ia64__) || (defined(__x86_64__) && !defined(__x86_64_x32__) && !defined(__AV_LLP64)) || defined(__s390x__)
+#if defined(__mips64__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppa64__) || defined(__arm64__) || defined(__powerpc64__) || defined(__ia64__) || (defined(__x86_64__) && !defined(__x86_64_x32__) && !defined(__AV_LLP64)) || defined(__s390x__) || defined(__riscv64__)
 /* ‘long long’ and ‘long’ are identical. */
 #define __av_longlong		__av_long
 #define __av_ulonglong		__av_ulong
@@ -398,7 +473,7 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 /* ‘long long’ fits in __avword. */
 #define __av_longlong			__av_word
 #define __av_ulonglong(LIST,VAL)	__av_word(LIST,(unsigned long long)(VAL))
-#elif defined(__i386__) || defined(__m68k__) || (defined(__mips__) && !defined(__mipsn32__) && !defined(__mips64__)) || (defined(__sparc__) && !defined(__sparc64__)) || (defined(__hppa__) && !defined(__hppa64__)) || defined(__arm__) || defined(__armhf__) || defined(__powerpc__) || defined(__x86_64_x32__) || (defined(__s390__) && !defined(__s390x__))
+#elif defined(__i386__) || defined(__m68k__) || (defined(__mips__) && !defined(__mipsn32__) && !defined(__mips64__)) || (defined(__sparc__) && !defined(__sparc64__)) || (defined(__hppa__) && !defined(__hppa64__)) || defined(__arm__) || defined(__armhf__) || defined(__powerpc__) || defined(__x86_64_x32__) || (defined(__s390__) && !defined(__s390x__)) || defined(__riscv32__)
 /* ‘long long’s are passed embedded on the arg stack. */
 #define __av_longlong(LIST,VAL)		__av_arg_longlong(LIST,long long,VAL)
 #define __av_ulonglong(LIST,VAL)	__av_arg_longlong(LIST,unsigned long long,VAL)
@@ -411,7 +486,7 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
     ((TYPE*)(LIST).aptr)[-1] = (TYPE)(VAL),				\
     0))
 #endif
-#if defined(__mips__) || (defined(__sparc__) && !defined(__sparc64__)) || (defined(__hppa__) && !defined(__hppa64__)) || defined(__arm__) || defined(__armhf__) || defined(__powerpc_sysv4__) || defined(__x86_64_x32__) || (defined(__s390__) && !defined(__s390x__))
+#if defined(__mips__) || (defined(__sparc__) && !defined(__sparc64__)) || (defined(__hppa__) && !defined(__hppa64__)) || defined(__arm__) || defined(__armhf__) || defined(__powerpc_sysv4__) || defined(__x86_64_x32__) || (defined(__s390__) && !defined(__s390x__)) || defined(__riscv32__)
 /* ‘long long’s have alignment 4 or 8. */
 #if defined(__mips__)
 #define __av_arg_longlong(LIST,TYPE,VAL)				\
@@ -473,8 +548,8 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #endif
 #if defined(__powerpc_sysv4__)
 #define __av_arg_longlong(LIST,TYPE,VAL)				\
-  ((((LIST).ianum + sizeof(TYPE)/sizeof(__avword)+__AV_alignof(TYPE)/sizeof(__avword)-1) & -(intptr_t)(__AV_alignof(TYPE)/sizeof(__avword))) <= __AV_IARG_NUM \
-   ? ((LIST).ianum = (((LIST).ianum + sizeof(TYPE)/sizeof(__avword)+__AV_alignof(TYPE)/sizeof(__avword)-1) & -(intptr_t)(__AV_alignof(TYPE)/sizeof(__avword))), \
+  ((((LIST).ianum + sizeof(TYPE)/sizeof(__avrword)+__AV_alignof(TYPE)/sizeof(__avrword)-1) & -(intptr_t)(__AV_alignof(TYPE)/sizeof(__avrword))) <= __AV_IARG_NUM \
+   ? ((LIST).ianum = (((LIST).ianum + sizeof(TYPE)/sizeof(__avrword)+__AV_alignof(TYPE)/sizeof(__avrword)-1) & -(intptr_t)(__AV_alignof(TYPE)/sizeof(__avrword))), \
       ((TYPE*)&(LIST).iargs[(LIST).ianum])[-1] = (TYPE)(VAL),		\
       0)								\
    : ((LIST).ianum = __AV_IARG_NUM,					\
@@ -487,14 +562,14 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 #if defined(__x86_64_x32__)
 #define __av_arg_longlong(LIST,TYPE,VAL)				\
   ((LIST).ianum < __AV_IARG_NUM						\
-   ? ((LIST).iargs[(LIST).ianum++] = (__avword)(TYPE)(VAL), 0)		\
+   ? ((LIST).iargs[(LIST).ianum++] = (__avrword)(TYPE)(VAL), 0)		\
    : __av_word(LIST,(TYPE)(VAL)))
 #endif
 #if (defined(__s390__) && !defined(__s390x__))
 /* Within the arg stack, the alignment is only 4, not 8. */
 #define __av_arg_longlong(LIST,TYPE,VAL)				\
-  ((LIST).ianum + (sizeof(TYPE)+sizeof(__avword)-1)/sizeof(__avword) <= __AV_IARG_NUM \
-   ? ((LIST).ianum += (sizeof(TYPE)+sizeof(__avword)-1)/sizeof(__avword), \
+  ((LIST).ianum + (sizeof(TYPE)+sizeof(__avrword)-1)/sizeof(__avrword) <= __AV_IARG_NUM \
+   ? ((LIST).ianum += (sizeof(TYPE)+sizeof(__avrword)-1)/sizeof(__avrword), \
       ((TYPE*)&(LIST).iargs[(LIST).ianum])[-1] = (TYPE)(VAL),		\
       0)								\
    : ((LIST).ianum = __AV_IARG_NUM,					\
@@ -504,6 +579,16 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
         ((TYPE*)(LIST).aptr)[-1] = (TYPE)(VAL),				\
         0))))
 #endif
+#if defined(__riscv32__)
+/* Within the arg stack, the alignment is only 4, not 8. Also, the argument
+   may be put into one word in registers and one word on the stack. */
+#define __av_arg_longlong(LIST,TYPE,VAL)				\
+  (((__avword*)(((uintptr_t)(LIST).aptr+sizeof(TYPE)+sizeof(__avword)-1) & -(intptr_t)sizeof(__avword)) > (LIST).eptr \
+    ? -1 :								\
+    ((LIST).aptr = (__avword*)(((uintptr_t)(LIST).aptr+sizeof(TYPE)+sizeof(__avword)-1) & -(intptr_t)sizeof(__avword)), \
+     ((TYPE*)(LIST).aptr)[-1] = (TYPE)(VAL),				\
+     0)))
+#endif
 #endif
 #endif
 
@@ -654,15 +739,21 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
   ((LIST).aptr <= (LIST).eptr						\
    ? -1 :								\
    ((LIST).aptr--,							\
+    ((LIST).aptr >= &(LIST).args_end[-4]				\
+     ? ((LIST).farg_mask |= (unsigned int)1 << ((LIST).args_end - (LIST).aptr - 1), 0) \
+     : 0),								\
     *(float*)(LIST).aptr = (float)(VAL),				\
     0))
 
 #define _av_double(LIST,VAL)						\
   ((__avword*)(((uintptr_t)(LIST).aptr-sizeof(double)) & -(intptr_t)sizeof(double)) < (LIST).eptr \
-    ? -1 :								\
-    ((LIST).aptr = (__avword*)(((uintptr_t)(LIST).aptr-sizeof(double)) & -(intptr_t)sizeof(double)), \
-     *(double*)(LIST).aptr = (double)(VAL),				\
-     0))
+   ? -1 :								\
+   ((LIST).aptr = (__avword*)(((uintptr_t)(LIST).aptr-sizeof(double)) & -(intptr_t)sizeof(double)), \
+    ((LIST).aptr >= &(LIST).args_end[-4]				\
+     ? ((LIST).darg_mask |= (unsigned int)1 << ((LIST).args_end - (LIST).aptr - 1), 0) \
+     : 0),								\
+    *(double*)(LIST).aptr = (double)(VAL),				\
+    0))
 
 #endif
 
@@ -742,9 +833,11 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
 
 #endif
 
-#if defined(__arm64__)
+#if defined(__arm64__) || defined(__riscv64__)
 
-/* Up to __AV_FARG_NUM float or double args can be passed in float registers. */
+/* Up to __AV_FARG_NUM float or double args can be passed in float registers.
+   The remaining float or double args are passed in the general-purpose
+   argument sequence (first the integer registers, then the stack.) */
 #define _av_float(LIST,VAL)						\
   ((LIST).fanum < __AV_FARG_NUM						\
    ? ((LIST).fargs[(LIST).fanum] = (float)(VAL),			\
@@ -758,16 +851,16 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
        0)))
 
 #define _av_double(LIST,VAL)						\
-  (((LIST).fanum < __AV_FARG_NUM					\
-    ? ((LIST).dargs[(LIST).fanum] = (double)(VAL),			\
-       (LIST).darg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
-       (LIST).fanum++,							\
-       0)								\
-    : ((LIST).aptr >= (LIST).eptr					\
-       ? -1 :								\
-       ((*(double*)(LIST).aptr = (double)(VAL)),			\
-        (LIST).aptr++,							\
-        0))))
+  ((LIST).fanum < __AV_FARG_NUM						\
+   ? ((LIST).dargs[(LIST).fanum] = (double)(VAL),			\
+      (LIST).darg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
+      (LIST).fanum++,							\
+      0)								\
+   : ((LIST).aptr >= (LIST).eptr					\
+      ? -1 :								\
+      ((*(double*)(LIST).aptr = (double)(VAL)),				\
+       (LIST).aptr += sizeof(double)/sizeof(__avword),			\
+       0)))
 
 #endif
 
@@ -1001,16 +1094,49 @@ typedef int __av_alist_verify[2*(__AV_ALIST_SIZE_BOUND - (int)sizeof(__av_alist)
        0)))
 
 #define _av_double(LIST,VAL)						\
-  (((LIST).fanum < __AV_FARG_NUM					\
-    ? ((LIST).dargs[(LIST).fanum] = (double)(VAL),			\
-       (LIST).darg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
-       (LIST).fanum++,							\
-       0)								\
-    : ((LIST).aptr >= (LIST).eptr					\
-       ? -1 :								\
-       ((*(double*)(LIST).aptr = (double)(VAL)),			\
-        (LIST).aptr++,							\
-        0))))
+  ((LIST).fanum < __AV_FARG_NUM						\
+   ? ((LIST).dargs[(LIST).fanum] = (double)(VAL),			\
+      (LIST).darg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
+      (LIST).fanum++,							\
+      0)								\
+   : ((LIST).aptr >= (LIST).eptr					\
+      ? -1 :								\
+      ((*(double*)(LIST).aptr = (double)(VAL)),				\
+       (LIST).aptr++,							\
+       0)))
+
+#endif
+
+#if defined(__riscv32__)
+
+/* Up to __AV_FARG_NUM float or double args can be passed in float registers.
+   The remaining float or double args are passed in the general-purpose
+   argument sequence (first the integer registers, then the stack.) */
+#define _av_float(LIST,VAL)						\
+  ((LIST).fanum < __AV_FARG_NUM						\
+   ? ((LIST).fargs[(LIST).fanum] = (float)(VAL),			\
+      (LIST).farg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
+      (LIST).fanum++,							\
+      0)								\
+   : ((LIST).aptr >= (LIST).eptr					\
+      ? -1 :								\
+      ((*(float*)(LIST).aptr) = (float)(VAL),				\
+       (LIST).aptr++,							\
+       0)))
+
+#define _av_double(LIST,VAL)						\
+  ((LIST).fanum < __AV_FARG_NUM						\
+   ? ((LIST).dargs[(LIST).fanum] = (double)(VAL),			\
+      (LIST).darg_mask |= ((unsigned int) 1) << (LIST).fanum,		\
+      (LIST).fanum++,							\
+      0)								\
+   : ((LIST).aptr + 2 > (LIST).eptr					\
+      ? -1 :								\
+      ((LIST).aptr += 2,						\
+       (LIST).tmp._double = (double)(VAL),				\
+       (LIST).aptr[-2] = (LIST).tmp.words[0],				\
+       (LIST).aptr[-1] = (LIST).tmp.words[1],				\
+       0)))
 
 #endif
 
@@ -1269,7 +1395,8 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
 #endif
 #if defined(__hppa__) && !defined(__hppa64__)
 /* Structures <= 8 bytes are passed as embedded copies on the arg stack.
- * Big structures are passed as pointers to caller-made local copies.
+ * Big structures are passed as pointers (to caller-made local copies
+ * with GCC >= 8, without copy otherwise).
  */
 #define __av_struct(LIST,TYPE_SIZE,TYPE_ALIGN,VAL)			\
   ((TYPE_SIZE) > 8							\
@@ -1298,9 +1425,9 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
  */
 #define __av_struct(LIST,TYPE_SIZE,TYPE_ALIGN,VAL)			\
   ((TYPE_SIZE) <= 16							\
-   ? ((((LIST).ianum*sizeof(__avword)+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) <= __AV_IARG_NUM*sizeof(__avword) \
+   ? ((((LIST).ianum*sizeof(__avrword)+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) <= __AV_IARG_NUM*sizeof(__avrword) \
       ? (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)&(LIST).iargs[(LIST).ianum],VAL), \
-         (LIST).ianum += (((((TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) + sizeof(__avword)-1) & -(intptr_t)sizeof(__avword))/sizeof(__avword), \
+         (LIST).ianum += (((((TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) + sizeof(__avrword)-1) & -(intptr_t)sizeof(__avrword))/sizeof(__avrword), \
          0)								\
       : ((__avword*)(((uintptr_t)(LIST).aptr+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) > (LIST).eptr \
          ? -1 :								\
@@ -1313,13 +1440,13 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
          > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
          ? -1 :								\
          (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)(LIST).eptr,VAL), \
-          (LIST).iargs[(LIST).ianum++] = (__avword)(LIST).eptr,		\
+          (LIST).iargs[(LIST).ianum++] = (__avrword)(LIST).eptr,	\
           0))								\
-      : (++(LIST).aptr							\
+      : (((LIST).aptr += sizeof(void*)/sizeof(__avword))		\
          > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
          ? -1 :								\
          (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)(LIST).eptr,VAL), \
-          (LIST).aptr[-1] = (__avword)(LIST).eptr,			\
+          ((void**)(LIST).aptr)[-1] = (LIST).eptr,			\
           0))))
 #endif
 #if defined(__powerpc_sysv4__)
@@ -1333,7 +1460,7 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
        > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
        ? -1 :								\
        (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)(LIST).eptr,VAL),	\
-        (LIST).iargs[(LIST).ianum++] = (__avword)(LIST).eptr,		\
+        (LIST).iargs[(LIST).ianum++] = (__avrword)(LIST).eptr,		\
         0))								\
     : (++(LIST).aptr							\
        > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
@@ -1350,9 +1477,9 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
    on the arg stack. */
 #define __av_struct(LIST,TYPE_SIZE,TYPE_ALIGN,VAL)				\
   (((TYPE_SIZE) <= 2*sizeof(__avword)						\
-    && (LIST).ianum + ((TYPE_SIZE) + sizeof(__avword)-1) / sizeof(__avword) <= __AV_IARG_NUM) \
+    && (LIST).ianum + ((TYPE_SIZE) + sizeof(__avrword)-1) / sizeof(__avrword) <= __AV_IARG_NUM) \
    ? (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)&(LIST).iargs[(LIST).ianum],VAL), \
-      (LIST).ianum += ((TYPE_SIZE) + sizeof(__avword)-1) / sizeof(__avword),	\
+      (LIST).ianum += ((TYPE_SIZE) + sizeof(__avrword)-1) / sizeof(__avrword),	\
       0)									\
    : ((__avword*)((uintptr_t)(LIST).aptr + (((TYPE_SIZE)+__av_struct_alignment(TYPE_ALIGN)-1) & -(intptr_t)__av_struct_alignment(TYPE_ALIGN))) > (LIST).eptr \
       ? -1 :									\
@@ -1384,8 +1511,8 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
  */
 #define __av_struct(LIST,TYPE_SIZE,TYPE_ALIGN,VAL)			\
   ((TYPE_SIZE) == 1 || (TYPE_SIZE) == 2 || (TYPE_SIZE) == 4 || (TYPE_SIZE) == 8	\
-   ? ((((LIST).ianum*sizeof(__avword)+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) <= __AV_IARG_NUM*sizeof(__avword) \
-      ? ((LIST).ianum += (((((TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) + sizeof(__avword)-1) & -(intptr_t)sizeof(__avword))/sizeof(__avword), \
+   ? ((((LIST).ianum*sizeof(__avrword)+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) <= __AV_IARG_NUM*sizeof(__avrword) \
+      ? ((LIST).ianum += (((((TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) + sizeof(__avrword)-1) & -(intptr_t)sizeof(__avrword))/sizeof(__avrword), \
          __av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)((uintptr_t)&(LIST).iargs[(LIST).ianum]-(TYPE_SIZE)),VAL), \
          0)								\
       : ((__avword*)(((uintptr_t)(LIST).aptr+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) > (LIST).eptr \
@@ -1399,7 +1526,7 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
          > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
          ? -1 :								\
          (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)(LIST).eptr,VAL), \
-          (LIST).iargs[(LIST).ianum++] = (__avword)(LIST).eptr,		\
+          (LIST).iargs[(LIST).ianum++] = (__avrword)(LIST).eptr,	\
           0))								\
       : (++(LIST).aptr							\
          > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
@@ -1408,6 +1535,24 @@ extern void avcall_structcpy (void* dest, const void* src, unsigned long size, u
           (LIST).aptr[-1] = (__avword)(LIST).eptr,			\
           0))))
 #endif
+#if defined(__riscv32__) || defined(__riscv64__)
+/* Structures <= 16 bytes are passed as embedded copies on the arg stack.
+ * Big structures are passed as pointers to caller-made local copies.
+ */
+#define __av_struct(LIST,TYPE_SIZE,TYPE_ALIGN,VAL)			\
+  ((TYPE_SIZE) <= 2*sizeof(__avword)					\
+   ? ((__avword*)(((uintptr_t)(LIST).aptr+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) > (LIST).eptr \
+      ? -1 :								\
+      (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)((((uintptr_t)(LIST).aptr+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) - (TYPE_SIZE)),VAL), \
+       (LIST).aptr = (__avword*)(((((uintptr_t)(LIST).aptr+(TYPE_SIZE)+(TYPE_ALIGN)-1) & -(intptr_t)(TYPE_ALIGN)) +sizeof(__avword)-1) & -(intptr_t)sizeof(__avword)), \
+       0))								\
+   : (++(LIST).aptr							\
+      > ((LIST).eptr = (__avword*)((uintptr_t)(LIST).eptr - (((TYPE_SIZE)+7)&-8)))\
+      ? -1 :								\
+      (__av_struct_copy(TYPE_SIZE,TYPE_ALIGN,(void*)(LIST).eptr,VAL),	\
+       (LIST).aptr[-1] = (__avword)(LIST).eptr,				\
+       0)))
+#endif
 
 
 #endif /* _AVCALL_INTERNAL_H */
