@@ -1,6 +1,6 @@
 /*
  * Copyright 1993-1995 Bill Triggs <Bill.Triggs@inrialpes.fr>
- * Copyright 1995-2017 Bruno Haible <bruno@clisp.org>
+ * Copyright 1995-2021 Bruno Haible <bruno@clisp.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _AVCALL_H
@@ -32,6 +32,7 @@
 
 
 /* Max # words in argument-list and temporary structure storage.
+ * If defined(__hppa__) && !defined(__hppa64__), this must be a multiple of 2.
  */
 #ifndef __AV_ALIST_WORDS
 #define __AV_ALIST_WORDS  256
@@ -39,7 +40,7 @@
 
 /* Determine whether the current ABI is LLP64
    ('long' = 32-bit, 'long long' = 'void*' = 64-bit). */
-#if defined(__x86_64__) && (defined(_WIN32) || defined(__WIN32__)) && !defined(__CYGWIN__)
+#if defined(__x86_64__) && defined(_WIN32) && !defined(__CYGWIN__)
 #define __AV_LLP64 1
 #endif
 
@@ -61,17 +62,25 @@ template <class type> struct __AV_alignof_helper { char __slot1; type __slot2; }
 extern "C" {
 #endif
 
-/* C builtin types.
+/* __avword represents a single word that can be pushed on the stack.
+ * __avrword represents a general-purpose register.
  */
-#if defined(__mipsn32__) || defined(__x86_64_x32__) || defined(__AV_LLP64)
+#if defined(__arm64__) && defined(__APPLE__) && defined(__MACH__)
+typedef int __avword;
+typedef long __avrword;
+#elif defined(__mipsn32__) || defined(__x86_64_x32__) || defined(__AV_LLP64)
 typedef long long __avword;
+typedef long long __avrword;
 #else
 typedef long __avword;
+typedef long __avrword;
 #endif
 
+/* C builtin types.
+ */
 enum __AVtype
 {
-  __AVword,
+  __AVword, /* just a dummy, for binary backward compatibility */
   __AVvoid,
   __AVchar,
   __AVschar,
@@ -139,13 +148,13 @@ enum __AV_alist_flags
 #if defined(__sparc__) && !defined(__sparc64__) && defined(__sun) && (defined(__SUNPRO_C) || defined(__SUNPRO_CC)) /* SUNWspro cc or CC */
 				  __AV_SUNPROCC_STRUCT_RETURN,
 #else
-#if (defined(__i386__) && (defined(_WIN32) || defined(__CYGWIN__) || (defined(__MACH__) && defined(__APPLE__)) || defined(__FreeBSD__) || defined(__DragonFly__) || defined(__OpenBSD__))) || defined(__m68k__) || defined(__mipsn32__) || defined(__mips64__) || defined(__sparc64__) || defined(__hppa__) || defined(__hppa64__) || defined(__arm__) || defined(__armhf__) || defined(__arm64__) || defined(__powerpc64_elfv2__) || defined(__ia64__) || defined(__x86_64__)
+#if (defined(__i386__) && (defined(_WIN32) || defined(__CYGWIN__) || (defined(__MACH__) && defined(__APPLE__)) || defined(__FreeBSD__) || defined(__DragonFly__) || defined(__OpenBSD__))) || defined(__m68k__) || defined(__mipsn32__) || defined(__mips64__) || defined(__sparc64__) || defined(__hppa__) || defined(__hppa64__) || defined(__arm__) || defined(__armhf__) || defined(__arm64__) || defined(__powerpc64_elfv2__) || defined(__ia64__) || defined(__x86_64__) || defined(__riscv32__) || defined(__riscv64__)
 				  __AV_SMALL_STRUCT_RETURN |
 #endif
 #if defined(__GNUC__) && !((defined(__mipsn32__) || defined(__mips64__)) && ((__GNUC__ == 3 && __GNUC_MINOR__ >= 4) || (__GNUC__ > 3)))
 				  __AV_GCC_STRUCT_RETURN |
 #endif
-#if defined(__i386__) && (defined(_WIN32) || defined(__WIN32__)) && !defined(__CYGWIN__) /* native Windows */
+#if defined(__i386__) && defined(_WIN32) && !defined(__CYGWIN__) /* native Windows */
 				  __AV_MSVC_STRUCT_RETURN |
 #endif
 				  0,
@@ -205,7 +214,7 @@ enum __AV_alist_flags
 #if defined(__powerpc__) && !defined(__powerpc64__) && defined(_AIX) && !defined(__GNUC__) /* AIX 32-bit cc, xlc */
   __AV_STRUCT_ARGS		= __AV_AIXCC_STRUCT_ARGS,
 #else
-#if defined(__powerpc64__) && defined(_AIX) /* AIX 64-bit cc, xlc, gcc */
+#if defined(__powerpc64__) && defined(_AIX) /* AIX 64-bit cc, xlc, gcc, xlclang */
   __AV_STRUCT_ARGS		= __AV_AIXCC_STRUCT_ARGS,
 #else
 #if defined(__ia64__) && !(defined(__GNUC__) && (__GNUC__ >= 3))
@@ -235,7 +244,7 @@ enum __AV_alist_flags
    * before or after #including <avcall.h>.
    */
 #ifndef __AV_FLOAT_ARGS
-#if defined(__powerpc64__) && defined(_AIX) && !defined(__GNUC__) /* AIX 64-bit xlc */
+#if defined(__powerpc64__) && defined(_AIX) && (defined(__ibmxl__) || !defined(__GNUC__)) /* AIX 64-bit xlc, xlclang */
   __AV_FLOAT_ARGS		= __AV_AIXCC_FLOAT_ARGS,
 #else
   __AV_FLOAT_ARGS		= 0,
@@ -256,7 +265,7 @@ enum __AV_alist_flags
 #endif
 
   /* These are for internal use only */
-#if defined(__i386__) || defined(__m68k__) || defined(__mipsn32__) || defined(__mips64__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppa64__) || defined(__arm__) || defined(__armhf__) || defined(__arm64__) || defined(__powerpc__) || defined(__powerpc64__) || defined(__ia64__) || defined(__x86_64__) || (defined(__s390__) && !defined(__s390x__))
+#if defined(__i386__) || defined(__m68k__) || defined(__mipsn32__) || defined(__mips64__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppa64__) || defined(__arm__) || defined(__armhf__) || defined(__arm64__) || defined(__powerpc__) || defined(__powerpc64__) || defined(__ia64__) || defined(__x86_64__) || (defined(__s390__) && !defined(__s390x__)) || defined(__riscv64__)
   __AV_REGISTER_STRUCT_RETURN	= 1<<9,
 #endif
 
@@ -323,46 +332,46 @@ extern int avcall_overflown (av_alist* /* LIST */);
 #define __AV_START_FLAGS  \
   __AV_STRUCT_RETURN | __AV_FLOAT_RETURN | __AV_STRUCT_ARGS | __AV_FLOAT_ARGS | __AV_CLEANUP
 
-extern void avcall_start (av_alist* /* LIST */, __avword* /* LIST_ARGS */, __avword* /* LIST_ARGS_END */, __avword(* /* FUNC */)(), void* /* RADDR */, int /* RETTYPE */, int /* FLAGS */);
+extern void avcall_start (av_alist* /* LIST */, __avword* /* LIST_ARGS */, __avword* /* LIST_ARGS_END */, __avrword(* /* FUNC */)(), void* /* RADDR */, int /* RETTYPE */, int /* FLAGS */);
 
 #define av_start_void(LIST,FUNC)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),0,    __AVvoid,     __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),0,    __AVvoid,     __AV_START_FLAGS)
 #define av_start_char(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVchar,     __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVchar,     __AV_START_FLAGS)
 #define av_start_schar(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVschar,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVschar,    __AV_START_FLAGS)
 #define av_start_uchar(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVuchar,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVuchar,    __AV_START_FLAGS)
 #define av_start_short(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVshort,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVshort,    __AV_START_FLAGS)
 #define av_start_ushort(LIST,FUNC,RADDR)				\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVushort,   __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVushort,   __AV_START_FLAGS)
 #define av_start_int(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVint,      __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVint,      __AV_START_FLAGS)
 #define av_start_uint(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVuint,     __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVuint,     __AV_START_FLAGS)
 #define av_start_long(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVlong,     __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVlong,     __AV_START_FLAGS)
 #define av_start_ulong(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVulong,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVulong,    __AV_START_FLAGS)
 #define av_start_longlong(LIST,FUNC,RADDR)				\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVlonglong, __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVlonglong, __AV_START_FLAGS)
 #define av_start_ulonglong(LIST,FUNC,RADDR)				\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVulonglong,__AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVulonglong,__AV_START_FLAGS)
 #define av_start_float(LIST,FUNC,RADDR)					\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVfloat,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVfloat,    __AV_START_FLAGS)
 #define av_start_double(LIST,FUNC,RADDR)				\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVdouble,   __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVdouble,   __AV_START_FLAGS)
 #define av_start_ptr(LIST,FUNC,TYPE,RADDR)				\
-  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),RADDR,__AVvoidp,    __AV_START_FLAGS)
+  avcall_start(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),RADDR,__AVvoidp,    __AV_START_FLAGS)
 
-extern void avcall_start_struct (av_alist* /* LIST */, __avword* /* LIST_ARGS */, __avword* /* LIST_ARGS_END */, __avword(* /* FUNC */)(), size_t /* TYPE_SIZE */, int /* TYPE_SPLITTABLE */, void* /* RADDR */, int /* FLAGS */);
+extern void avcall_start_struct (av_alist* /* LIST */, __avword* /* LIST_ARGS */, __avword* /* LIST_ARGS_END */, __avrword(* /* FUNC */)(), size_t /* TYPE_SIZE */, int /* TYPE_SPLITTABLE */, void* /* RADDR */, int /* FLAGS */);
 
 #define av_start_struct(LIST,FUNC,TYPE,TYPE_SPLITTABLE,RADDR)		\
   _av_start_struct(LIST,FUNC,sizeof(TYPE),TYPE_SPLITTABLE,RADDR)
 /* Undocumented, but used by GNU clisp. */
 #define _av_start_struct(LIST,FUNC,TYPE_SIZE,TYPE_SPLITTABLE,RADDR)	\
-  avcall_start_struct(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avword(*)())(FUNC),TYPE_SIZE,TYPE_SPLITTABLE,RADDR,__AV_START_FLAGS)
+  avcall_start_struct(&(LIST),(LIST)._av_alist_flexarray._av_m_args,&(LIST)._av_alist_flexarray._av_m_args[__AV_ALIST_WORDS],(__avrword(*)())(FUNC),TYPE_SIZE,TYPE_SPLITTABLE,RADDR,__AV_START_FLAGS)
 
 
 /*
@@ -371,17 +380,21 @@ extern void avcall_start_struct (av_alist* /* LIST */, __avword* /* LIST_ARGS */
 
 /* integer argument types */
 
+extern int avcall_arg_int (av_alist* /* LIST */, int /* VAL */);
+extern int avcall_arg_uint (av_alist* /* LIST */, unsigned int /* VAL */);
+
+#define av_char(LIST,VAL)	avcall_arg_int(&(LIST),(char)(VAL))
+#define av_schar(LIST,VAL)	avcall_arg_int(&(LIST),(signed char)(VAL))
+#define av_short(LIST,VAL)	avcall_arg_int(&(LIST),(short)(VAL))
+#define av_int(LIST,VAL)	avcall_arg_int(&(LIST),(int)(VAL))
+#define av_uchar(LIST,VAL)	avcall_arg_uint(&(LIST),(unsigned char)(VAL))
+#define av_ushort(LIST,VAL)	avcall_arg_uint(&(LIST),(unsigned short)(VAL))
+#define av_uint(LIST,VAL)	avcall_arg_uint(&(LIST),(unsigned int)(VAL))
+
 extern int avcall_arg_long (av_alist* /* LIST */, long /* VAL */);
 extern int avcall_arg_ulong (av_alist* /* LIST */, unsigned long /* VAL */);
 
-#define av_char(LIST,VAL)	avcall_arg_long(&(LIST),(char)(VAL))
-#define av_schar(LIST,VAL)	avcall_arg_long(&(LIST),(signed char)(VAL))
-#define av_short(LIST,VAL)	avcall_arg_long(&(LIST),(short)(VAL))
-#define av_int(LIST,VAL)	avcall_arg_long(&(LIST),(int)(VAL))
 #define av_long(LIST,VAL)	avcall_arg_long(&(LIST),(long)(VAL))
-#define av_uchar(LIST,VAL)	avcall_arg_ulong(&(LIST),(unsigned char)(VAL))
-#define av_ushort(LIST,VAL)	avcall_arg_ulong(&(LIST),(unsigned short)(VAL))
-#define av_uint(LIST,VAL)	avcall_arg_ulong(&(LIST),(unsigned int)(VAL))
 #define av_ulong(LIST,VAL)	avcall_arg_ulong(&(LIST),(unsigned long)(VAL))
 
 extern int avcall_arg_ptr (av_alist* /* LIST */, void* /* VAL */);
